<?php
/*
* $Id: dir.libphp4.php, v1.1 2011.04.14 for PHP4+
* 
*  2011 Skurat Denis (a.k.a. Dionys)
* @contact: skiner@inbox.ru
**
* :: mkdirr			- Recursive make dir
* :: rmdirr			- Recursive remove dir with files
* :: nb_copy		- Non-block copy
* :: dirlistr		- Recursive dirlist class
* :: wb_rmdirr		- Recursive remove dir with files for winbinder
* :: wb_copy		- Non-block copy for winbinder (need initialized refresh() window function)
* :: wb_dirlistr	- Recursive dirlist class for winbinder
**
*/

/* Recursive make dir */
function mkdirr($dir, $mode = 0777)
{
	if (is_dir($dir) || @mkdir($dir, $mode))
	{
		return true;
	}
	if (!mkdirr(dirname($dir), $mode))
	{
		return false;
	}

	return @mkdir($dir, $mode);
}

/* Recursive remove dir with files */
function rmdirr($in_dir)
{
	if (substr($in_dir, strlen($in_dir) - 1, 1) == '/')
	{
		$in_dir = substr($in_dir, 0, -1);
	}
	chmod($in_dir, 0777);

	if ($dir_handle = @opendir($in_dir))
	{
		while ($file = readdir($dir_handle))
		{
			/* dir step-in */
			if (is_dir($in_dir.'/'.$file) && $file != '..' && $file != '.')
			{
				if (!rmdirr($in_dir.'/'.$file))
				{
					return false;
				}
			}
			/* search file */
			else if (is_file($in_dir.'/'.$file) && $file != '..' && $file != '.')
			{
				if (!unlink($in_dir.'/'.$file))
				{
					return false;
				}
			}
		}
		closedir($dir_handle);

		if (!rmdir($in_dir))
		{
			return false;
		}
		else
		{
			return true;
		}
	}
	else
	{
		return false;
	}
}

/* Non-block copy */
function nb_copy ($source, $target, $chunkConst = array())
{
	$targetTmp = substr($target, 0, -strlen(basename($target)));
	if (!mkdirr($targetTmp))
	{
		return false;
	}

	$hSource = @fopen($source, 'r');
	$hTarget = @fopen($target, 'w');

	if ($hSource && $hTarget)
	{
		stream_set_timeout($hSource, 5);
		stream_set_timeout($hTarget, 5);

		$hSourceStat = fstat($hSource);
		$chunkSize = intval($hSourceStat['size'])/100;
		if ($chunkSize > $chunkConst['max'])
		{
			$chunkSize = $chunkConst['max'];
		}
		else if ($chunkSize < $chunkConst['min'])
		{
			$chunkSize = $chunkConst['min'];
		}

		while (!feof($hSource))
		{
			$chunk = fread($hSource, $chunkSize);
			fwrite($hTarget, $chunk);
		}

		fclose($hSource);
		fflush($hTarget);
		fclose($hTarget);

		return true;
	}
	else
	{
		return false;
	}
}

/* Recursive dirlist class */
class dirlistr
{
	var $dirlist	= array();
	var $count		= 0;

	/* search function */
	function go_dirlistr($in_dir)
	{
		if (substr($in_dir, strlen($in_dir) - 1, 1) == '/')
		{
			$in_dir = substr($in_dir, 0, -1);
		}

		if ($dir_handle = @opendir($in_dir))
		{
			while($file = readdir($dir_handle))
			{
				/* dir step-in */
				if (is_dir($in_dir.'/'.$file) && $file != '..' && $file != '.')
				{
					if (!$this->go_dirlistr($in_dir.'/'.$file))
					{
						closedir($dir_handle);
						return false;
					}
				}
				/* search file */
				else if (is_file($in_dir.'/'.$file) && $file != '..' && $file != '.')
				{
					$this->dirlist[$this->count] = $in_dir.'/'.$file;
					$this->count ++;
				}
			}
			closedir($dir_handle);
			return true;
		}
		else
		{
			return false;
		}
	}
}

/* Recursive remove dir with files for winbinder */
function wb_rmdirr($in_dir, $wbobject = 0)
{
	if (substr($in_dir, strlen($in_dir) - 1, 1) == '/')
	{
		$in_dir = substr($in_dir, 0, -1);
	}

	if ($dir_handle = @opendir($in_dir))
	{
		while ($file = readdir($dir_handle))
		{
			/* dir step-in */
			if (is_dir($in_dir.'/'.$file) && $file != '..' && $file != '.')
			{
				chmod($in_dir.'/'.$file, 0777);
				wb_rmdirr($in_dir.'/'.$file, $wbobject);
			}
			/* search file */
			else if (is_file($in_dir.'/'.$file) && $file != '..' && $file != '.')
			{
				chmod($in_dir.'/'.$file, 0777);
				@unlink($in_dir.'/'.$file);

				wb_set_text($wbobject, calc_label_echo('', $in_dir.'/'.$file));
				refresh();
			}
		}
		closedir($dir_handle);
		@rmdir($in_dir);
	}
}

/* Non-block copy for winbinder (need initialized refresh() window function) */
function wb_copy ($source, $target, $chunkConst = array() , $wbobjects = array())
{
	$mainwin = $wbobjects['mainwin'];
	$statusbar = $wbobjects['statusbar'];
	$textlabel = $wbobjects['textlabel'];
	$pbar = $wbobjects['pbar'];

	$targetTmp = substr($target, 0, -strlen(basename($target)));
	if (mkdirr($targetTmp))
	{
		wb_set_text($textlabel, ' .');
	}
	else
	{
		wb_set_text($textlabel, '  .');
		wb_set_text($statusbar, '!');
		wb_message_box	($mainwin,
						'  :'."\n".
						$targetTmp.
						NULL,
						APPNAME, WBC_STOP);

		return false;
	}

	$hSource = @fopen($source, 'r');
	$hTarget = @fopen($target, 'w');

	if ($hSource && $hTarget)
	{
		stream_set_timeout($hSource, 5);
		stream_set_timeout($hTarget, 5);

		wb_set_text($textlabel, ' ...');

		$hTargetSize = 0;
		$hSourceStat = fstat($hSource);
		$chunkSize = intval($hSourceStat['size'])/100;
		if ($chunkSize > $chunkConst['max'])
		{
			$chunkSize = $chunkConst['max'];
		}
		else if ($chunkSize < $chunkConst['min'])
		{
			$chunkSize = $chunkConst['min'];
		}

		wb_set_range($pbar, 0, $hSourceStat['size']);

		while (!feof($hSource))
		{
			$chunk = fread($hSource, $chunkSize);
			fwrite($hTarget, $chunk);
			$hTargetSize += strlen($chunk);

			wb_set_value($pbar, $hTargetSize);
			if (!refresh())
			{
				fclose($hSource);
				fflush($hTarget);
				fclose($hTarget);
				return false;
			}
		}

		wb_set_value($pbar, $hSourceStat['size']);

		fclose($hSource);
		fflush($hTarget);
		fclose($hTarget);

		return true;
	}
	else
	{
		wb_set_text($textlabel, '  .');
		wb_set_text($statusbar, '!');
		wb_message_box	($mainwin,
						'  :'."\n".
						$target.
						NULL,
						APPNAME, WBC_STOP);
		return false;
	}
}

/* Recursive dirlist class for winbinder */
class wb_dirlistr
{
	var $dirlist	= array();
	var $count		= 0;
	var $wbobject	= 0;

	/* search function */
	function go_dirlistr($in_dir)
	{
		if (substr($in_dir, strlen($in_dir) - 1, 1) == '/')
		{
			$in_dir = substr($in_dir, 0, -1);
		}

		if ($dir_handle = @opendir($in_dir))
		{
			while($file = readdir($dir_handle))
			{
				/* dir step-in */
				if (is_dir($in_dir.'/'.$file) && $file != '..' && $file != '.')
				{
					if (!$this->go_dirlistr($in_dir.'/'.$file))
					{
						closedir($dir_handle);
						return false;
					}
				}
				/* search file */
				else if (is_file($in_dir.'/'.$file) && $file != '..' && $file != '.')
				{
					$this->dirlist[$this->count] = $in_dir.'/'.$file;
					$this->count ++;

					wb_set_text($this->wbobject, calc_label_echo('', $in_dir.'/'.$file));
					if (!refresh())
					{
						closedir($dir_handle);
						return false;
					}
				}
			}
			closedir($dir_handle);
			return true;
		}
		else
		{
			return false;
		}
	}
}
?>